// city.cpp
// by Glenn G. Chappell
// October 2003
//
// For CS 381
// "City" scene

#include <iostream>
using std::cerr;
using std::endl;
#include <string>
using std::string;
#include <stdlib.h>  // Some versions of MS-Vis C++ have broken <cstdlib>
//using std::exit;
#include <GL/glut.h> // GLUT stuff - includes OpenGL headers as well


// Global variables
// Window/viewport
const int startwinsize = 400; // Starting window width & height, in pixels
const double neardist = 1.;   // Near & far clipping distances
const double fardist = 5.;

// Keyboard
const int ESCKEY = 27;        // ASCII value of escape character


// printbitmap
// Prints the given string at the given raster position
//  using GLUT bitmap fonts.
// You probably don't want any rotations in the model/view
//  transformation when calling this function.
void printbitmap(const string msg, double x, double y)
{
   glRasterPos2d(x, y);
   for (string::const_iterator ii = msg.begin();
        ii != msg.end();
        ++ii)
   {
      glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *ii);
   }
}


// drawbox
// Draws a multi-colored cube.
// Center is (0,0,0), side length is 2.
// No back face is drawn.
void drawbox()
{
   glBegin(GL_QUADS);
      // Front face
      glColor3d(0.9, 0.1, 0.1);
      glVertex3d(-1., -1.,  1.);
      glVertex3d( 1., -1.,  1.);
      glVertex3d( 1.,  1.,  1.);
      glVertex3d(-1.,  1.,  1.);

      //// Back face
      //glColor3d(0.1, 0.9, 0.9);
      //glVertex3d(-1.,  1., -1.);
      //glVertex3d( 1.,  1., -1.);
      //glVertex3d( 1., -1., -1.);
      //glVertex3d(-1., -1., -1.);

      // Top face
      glColor3d(0.9, 0.9, 0.1);
      glVertex3d(-1.,  1.,  1.);
      glVertex3d( 1.,  1.,  1.);
      glVertex3d( 1.,  1., -1.);
      glVertex3d(-1.,  1., -1.);

      // Bottom face
      glColor3d(0.1, 0.1, 0.9);
      glVertex3d(-1., -1., -1.);
      glVertex3d( 1., -1., -1.);
      glVertex3d( 1., -1.,  1.);
      glVertex3d(-1., -1.,  1.);

      // Right face
      glColor3d(0.1, 0.9, 0.1);
      glVertex3d( 1., -1.,  1.);
      glVertex3d( 1., -1., -1.);
      glVertex3d( 1.,  1., -1.);
      glVertex3d( 1.,  1.,  1.);

      // Left face
      glColor3d(0.9, 0.1, 0.9);
      glVertex3d(-1.,  1.,  1.);
      glVertex3d(-1.,  1., -1.);
      glVertex3d(-1., -1., -1.);
      glVertex3d(-1., -1.,  1.);
   glEnd();
}


// drawcity
// Draws a 2x2 square, in the x,y plane,
//  convered with cubical blocks on the +z side.
void drawcity()
{
   const int halfside = 8;
      // Half of the side of the square in the coord system we draw it in.
      // "Buildings" are drawn at unit intervals from -halfside+1/2 to halfside-1/2.
      // We scale the square down so that it goes from -1 to 1 in both x & y.
   glPushMatrix();
      glScaled(1./halfside, 1./halfside, 1./halfside);

      // Draw gray square
      glColor3d(0.7, 0.7, 0.7);
      glBegin(GL_QUADS);
         glVertex3d(-halfside, -halfside, 0.);
         glVertex3d( halfside, -halfside, 0.);
         glVertex3d( halfside,  halfside, 0.);
         glVertex3d(-halfside,  halfside, 0.);
      glEnd();

      // Draw "buildings": boxes with side 1/2, spaced at intervals of 1 unit.
      int i, j;  // loop counters. Actual (x, y) of building is (i+0.5, j+0.5).
      for (i=-halfside; i<halfside; ++i)
      {
         for (j=-halfside; j<halfside; ++j)
         {
            // Draw a multi-colored box with side 1/2. 
            glPushMatrix();
               glTranslated(0., 0., 0.002);     // Raise it a hair above the ground
               glTranslated(i+0.5, j+0.5, 0.);  // Position it in the city
               glTranslated(0., 0., 0.25);      // Put back side right on the ground
               glScaled(0.25, 0.25, 0.25);      // Give the building side 1/2
               drawbox();
            glPopMatrix();
         }
      }
   glPopMatrix();
}


// display
// The GLUT display function
void display()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   glPushMatrix();
      glTranslated(0., 0., -3.);
      drawcity();
   glPopMatrix();

   // Text documentation
   glDisable(GL_DEPTH_TEST);  // No depth test for text
   glMatrixMode(GL_PROJECTION);
   glPushMatrix();  // Save old projection matrix
      glLoadIdentity();
      gluOrtho2D(-1., 1., -1., 1.);  // old faithful
      glColor3d(0., 0., 0.);
      printbitmap("City", -0.9, 0.9);
      printbitmap("Esc     Quit", -0.9, 0.8);
   glPopMatrix();  // Restore old projection matrix
   glMatrixMode(GL_MODELVIEW);  // Always go back to model/view mode
   glEnable(GL_DEPTH_TEST);  // Back to "normal" for 3-D drawing

   glutSwapBuffers();
}


// reshape
// The GLUT reshape function
void reshape(int w, int h)
{
   glViewport(0, 0, w, h);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(45, double(w)/h, neardist, fardist);
   // When drawing text, we set up a temporary projection with
   //  gluOrtho2D, so we don't need to worry about exactly where
   //  left, right, bottom, and top are.

   glMatrixMode(GL_MODELVIEW);  // Always go back to modelview mode
}


// idle
// The GLUT idle function
void idle()
{
   // Print OpenGL errors, if there are any (for debugging)
   if (GLenum err = glGetError())
   {
      cerr << "OpenGL ERROR: " << gluErrorString(err) << endl;
   }
}


// keyboard
// The GLUT keyboard function
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
   case ESCKEY:  // ESC: Quit
      exit(0);
      break;
   }
}


// init
// Initialization
// Called by main after window creation
void init()
{
   // General
   glClearColor(0.7, 0.8, 0.8, 0.0);  // Light blue-gray background
   glEnable(GL_DEPTH_TEST);           // 3-D so we need HSR
}


int main(int argc, char ** argv)
{
   // Initialize OpenGL/GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

   // Make a window
   glutInitWindowSize(startwinsize, startwinsize);
   glutInitWindowPosition(50, 50);
   glutCreateWindow("CS 381 - City");

   // Initialize GL states & register callbacks
   init();
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutIdleFunc(idle);
   glutKeyboardFunc(keyboard);

   // Do something
   glutMainLoop();

   return 0;
}
